# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

"""
RWStl module, see official documentation at
https://www.opencascade.com/doc/occt-7.4.0/refman/html/package_rwstl.html
"""

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _RWStl
else:
    import _RWStl

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


class SwigPyIterator(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _RWStl.delete_SwigPyIterator

    def value(self) -> "PyObject *":
        return _RWStl.SwigPyIterator_value(self)

    def incr(self, n: "size_t"=1) -> "swig::SwigPyIterator *":
        return _RWStl.SwigPyIterator_incr(self, n)

    def decr(self, n: "size_t"=1) -> "swig::SwigPyIterator *":
        return _RWStl.SwigPyIterator_decr(self, n)

    def distance(self, x: "SwigPyIterator") -> "ptrdiff_t":
        return _RWStl.SwigPyIterator_distance(self, x)

    def equal(self, x: "SwigPyIterator") -> "bool":
        return _RWStl.SwigPyIterator_equal(self, x)

    def copy(self) -> "swig::SwigPyIterator *":
        return _RWStl.SwigPyIterator_copy(self)

    def next(self) -> "PyObject *":
        return _RWStl.SwigPyIterator_next(self)

    def __next__(self) -> "PyObject *":
        return _RWStl.SwigPyIterator___next__(self)

    def previous(self) -> "PyObject *":
        return _RWStl.SwigPyIterator_previous(self)

    def advance(self, n: "ptrdiff_t") -> "swig::SwigPyIterator *":
        return _RWStl.SwigPyIterator_advance(self, n)

    def __eq__(self, x: "SwigPyIterator") -> "bool":
        return _RWStl.SwigPyIterator___eq__(self, x)

    def __ne__(self, x: "SwigPyIterator") -> "bool":
        return _RWStl.SwigPyIterator___ne__(self, x)

    def __iadd__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator &":
        return _RWStl.SwigPyIterator___iadd__(self, n)

    def __isub__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator &":
        return _RWStl.SwigPyIterator___isub__(self, n)

    def __add__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator *":
        return _RWStl.SwigPyIterator___add__(self, n)

    def __sub__(self, *args) -> "ptrdiff_t":
        return _RWStl.SwigPyIterator___sub__(self, *args)
    def __iter__(self):
        return self

# Register SwigPyIterator in _RWStl:
_RWStl.SwigPyIterator_swigregister(SwigPyIterator)


def _dumps_object(klass):
    """ Overwrite default string output for any wrapped object.
    By default, __repr__ method returns something like:
    <OCC.Core.TopoDS.TopoDS_Shape; proxy of <Swig Object of type 'TopoDS_Shape *' at 0x02BB0758> >
    This is too much verbose.
    We prefer :
    <class 'gp_Pnt'>
    or
    <class 'TopoDS_Shape'>
    """
    klass_name = str(klass.__class__).split(".")[3].split("'")[0]
    repr_string = "<class '" + klass_name + "'"
# for TopoDS_Shape, we also look for the base type
    if klass_name == "TopoDS_Shape":
        if klass.IsNull():
            repr_string += ": Null>"
            return repr_string
        st = klass.ShapeType()
        types = {OCC.Core.TopAbs.TopAbs_VERTEX: "Vertex",
                 OCC.Core.TopAbs.TopAbs_SOLID: "Solid",
                 OCC.Core.TopAbs.TopAbs_EDGE: "Edge",
                 OCC.Core.TopAbs.TopAbs_FACE: "Face",
                 OCC.Core.TopAbs.TopAbs_SHELL: "Shell",
                 OCC.Core.TopAbs.TopAbs_WIRE: "Wire",
                 OCC.Core.TopAbs.TopAbs_COMPOUND: "Compound",
                 OCC.Core.TopAbs.TopAbs_COMPSOLID: "Compsolid"}
        repr_string += "; Type:%s" % types[st]        
    elif hasattr(klass, "IsNull"):
        if klass.IsNull():
            repr_string += "; Null"
    repr_string += ">"
    return repr_string


def process_exception(error: "Standard_Failure", method_name: "std::string", class_name: "std::string") -> "void":
    return _RWStl.process_exception(error, method_name, class_name)

from six import with_metaclass
import warnings
from OCC.Wrapper.wrapper_utils import Proxy, deprecated

import OCC.Core.Standard
import OCC.Core.NCollection
import OCC.Core.OSD
import OCC.Core.TCollection
import OCC.Core.Message
import OCC.Core.TColStd
import OCC.Core.Poly
import OCC.Core.gp
import OCC.Core.TColgp
import OCC.Core.TShort

from enum import IntEnum
from OCC.Core.Exception import *



class rwstl(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    @staticmethod
    def ReadAscii(*args) -> "opencascade::handle< Poly_Triangulation >":
        r"""
        Read triangulation from an ascii stl file in case of error, returns null handle.

        Parameters
        ----------
        thePath: OSD_Path
        theProgress: Message_ProgressRange,optional
        	default value is Message_ProgressRange()

        Returns
        -------
        opencascade::handle<Poly_Triangulation>

        """
        return _RWStl.rwstl_ReadAscii(*args)

    @staticmethod
    def ReadBinary(*args) -> "opencascade::handle< Poly_Triangulation >":
        r"""
        Read triangulation from a binary stl file in case of error, returns null handle.

        Parameters
        ----------
        thePath: OSD_Path
        theProgress: Message_ProgressRange,optional
        	default value is Message_ProgressRange()

        Returns
        -------
        opencascade::handle<Poly_Triangulation>

        """
        return _RWStl.rwstl_ReadBinary(*args)

    @staticmethod
    def ReadFile(*args) -> "opencascade::handle< Poly_Triangulation >":
        r"""
        Read specified stl file and returns its content as triangulation. in case of error, returns null handle.

        Parameters
        ----------
        theFile: OSD_Path
        aProgInd: Message_ProgressRange,optional
        	default value is Message_ProgressRange()

        Returns
        -------
        opencascade::handle<Poly_Triangulation>
        Read specified stl file and returns its content as triangulation. in case of error, returns null handle.

        Parameters
        ----------
        theFile: char *
        aProgInd: Message_ProgressRange,optional
        	default value is Message_ProgressRange()

        Returns
        -------
        opencascade::handle<Poly_Triangulation>

        """
        return _RWStl.rwstl_ReadFile(*args)

    @staticmethod
    def WriteAscii(*args) -> "Standard_Boolean":
        r"""
        Write the meshing in a file following the ascii format of an stl file. returns false if the cannot be opened;.

        Parameters
        ----------
        theMesh: Poly_Triangulation
        thePath: OSD_Path
        theProgress: Message_ProgressRange,optional
        	default value is Message_ProgressRange()

        Returns
        -------
        bool

        """
        return _RWStl.rwstl_WriteAscii(*args)

    @staticmethod
    def WriteBinary(*args) -> "Standard_Boolean":
        r"""
        Write triangulation to binary stl file. binary format of an stl file. returns false if the cannot be opened;.

        Parameters
        ----------
        theMesh: Poly_Triangulation
        thePath: OSD_Path
        theProgress: Message_ProgressRange,optional
        	default value is Message_ProgressRange()

        Returns
        -------
        bool

        """
        return _RWStl.rwstl_WriteBinary(*args)

    __repr__ = _dumps_object


    def __init__(self):
        _RWStl.rwstl_swiginit(self, _RWStl.new_rwstl())
    __swig_destroy__ = _RWStl.delete_rwstl

# Register rwstl in _RWStl:
_RWStl.rwstl_swigregister(rwstl)

def rwstl_ReadAscii(*args) -> "opencascade::handle< Poly_Triangulation >":
    r"""
    Read triangulation from an ascii stl file in case of error, returns null handle.

    Parameters
    ----------
    thePath: OSD_Path
    theProgress: Message_ProgressRange,optional
    	default value is Message_ProgressRange()

    Returns
    -------
    opencascade::handle<Poly_Triangulation>

    """
    return _RWStl.rwstl_ReadAscii(*args)

def rwstl_ReadBinary(*args) -> "opencascade::handle< Poly_Triangulation >":
    r"""
    Read triangulation from a binary stl file in case of error, returns null handle.

    Parameters
    ----------
    thePath: OSD_Path
    theProgress: Message_ProgressRange,optional
    	default value is Message_ProgressRange()

    Returns
    -------
    opencascade::handle<Poly_Triangulation>

    """
    return _RWStl.rwstl_ReadBinary(*args)

def rwstl_ReadFile(*args) -> "opencascade::handle< Poly_Triangulation >":
    r"""
    Read specified stl file and returns its content as triangulation. in case of error, returns null handle.

    Parameters
    ----------
    theFile: OSD_Path
    aProgInd: Message_ProgressRange,optional
    	default value is Message_ProgressRange()

    Returns
    -------
    opencascade::handle<Poly_Triangulation>
    Read specified stl file and returns its content as triangulation. in case of error, returns null handle.

    Parameters
    ----------
    theFile: char *
    aProgInd: Message_ProgressRange,optional
    	default value is Message_ProgressRange()

    Returns
    -------
    opencascade::handle<Poly_Triangulation>

    """
    return _RWStl.rwstl_ReadFile(*args)

def rwstl_WriteAscii(*args) -> "Standard_Boolean":
    r"""
    Write the meshing in a file following the ascii format of an stl file. returns false if the cannot be opened;.

    Parameters
    ----------
    theMesh: Poly_Triangulation
    thePath: OSD_Path
    theProgress: Message_ProgressRange,optional
    	default value is Message_ProgressRange()

    Returns
    -------
    bool

    """
    return _RWStl.rwstl_WriteAscii(*args)

def rwstl_WriteBinary(*args) -> "Standard_Boolean":
    r"""
    Write triangulation to binary stl file. binary format of an stl file. returns false if the cannot be opened;.

    Parameters
    ----------
    theMesh: Poly_Triangulation
    thePath: OSD_Path
    theProgress: Message_ProgressRange,optional
    	default value is Message_ProgressRange()

    Returns
    -------
    bool

    """
    return _RWStl.rwstl_WriteBinary(*args)


@classnotwrapped
class RWStl_Reader:
	pass









##Copyright 2017 Thomas Paviot (tpaviot@gmail.com)
##
##This file is part of pythonOCC.
##
##pythonOCC is free software: you can redistribute it and/or modify
##it under the terms of the GNU Lesser General Public License as published by
##the Free Software Foundation, either version 3 of the License, or
##(at your option) any later version.
##
##pythonOCC is distributed in the hope that it will be useful,
##but WITHOUT ANY WARRANTY; without even the implied warranty of
##MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##GNU Lesser General Public License for more details.
##
##You should have received a copy of the GNU Lesser General Public License
##along with pythonOCC.  If not, see <http://www.gnu.org/licenses/>.

import enum
from functools import reduce
import itertools
import math
import uuid
import sys

# pythreejs
try:
    from pythreejs import (CombinedCamera, BufferAttribute, BufferGeometry, Mesh,
                           LineSegmentsGeometry, LineMaterial, LineSegments2, AmbientLight,
                           DirectionalLight, Scene, OrbitControls, Renderer,
                           Picker, Group, GridHelper, Line,
                           ShaderMaterial, ShaderLib, LineBasicMaterial,
                           PointsMaterial, Points, make_text)
    from IPython.display import display, SVG
    from ipywidgets import HTML, HBox, VBox, Checkbox, Button, Layout, Dropdown, embed
    import numpy as np
except ImportError:
    error_log = """ Error You must install pythreejs/ipywidgets/numpy to run the jupyter notebook renderer.
If you installed pythonocc using conda, just type :
$ conda install -c conda-forge pythreejs"""
    print(error_log)
    sys.exit(0)


from OCC.Core.Bnd import Bnd_Box
from OCC.Core.BRepPrimAPI import BRepPrimAPI_MakeSphere
from OCC.Core.BRepBndLib import brepbndlib_Add
from OCC.Core.gp import gp_Pnt, gp_Dir
from OCC.Core.TopoDS import TopoDS_Compound
from OCC.Core.BRepBuilderAPI import BRepBuilderAPI_MakeVertex
from OCC.Core.BRep import BRep_Builder
from OCC.Core.Tesselator import ShapeTesselator

from OCC.Extend.TopologyUtils import (TopologyExplorer, is_edge, is_wire, discretize_edge,
                                      discretize_wire, get_type_as_string)
from OCC.Extend.ShapeFactory import (get_oriented_boundingbox,
                                     get_aligned_boundingbox,
                                     measure_shape_mass_center_of_gravity,
                                     recognize_face)
from OCC.Extend.DataExchange import export_shape_to_svg

#
# Util mathematical functions
#
def _add(vec1, vec2):
    return list(v1 + v2 for v1, v2 in zip(vec1, vec2))


def _explode(edge_list):
    return [[edge_list[i], edge_list[i + 1]] for i in range(len(edge_list) - 1)]


def _flatten(nested_dict):
    return [y for x in nested_dict for y in x]


def format_color(r, g, b):
    return '#%02x%02x%02x' % (r, g, b)


def _distance(v1, v2):
    return np.linalg.norm([x - y for x, y in zip(v1, v2)])


def _bool_or_new(val):
    return val if isinstance(val, bool) else val["new"]


def _opt(b1, b2):
    return (min(b1[0], b2[0]), max(b1[1], b2[1]), min(b1[2], b2[2]),
            max(b1[3], b2[3]), min(b1[4], b2[4]), max(b1[5], b2[5]))


def _shift(v, offset):
    return [x + o for x, o in zip(v, offset)]


# https://stackoverflow.com/questions/4947682/intelligently-calculating-chart-tick-positions
def _nice_number(value, round_=False):
    exponent = math.floor(math.log(value, 10))
    fraction = value / 10**exponent

    if round_:
        if fraction < 1.5:
            nice_fraction = 1.
        elif fraction < 3.:
            nice_fraction = 2.
        elif fraction < 7.:
            nice_fraction = 5.
        else:
            nice_fraction = 10.
    else:
        if fraction <= 1:
            nice_fraction = 1.
        elif fraction <= 2:
            nice_fraction = 2.
        elif fraction <= 5:
            nice_fraction = 5.
        else:
            nice_fraction = 10.

    return nice_fraction * 10**exponent

def _nice_bounds(axis_start, axis_end, num_ticks=10):
    axis_width = axis_end - axis_start
    if axis_width == 0:
        nice_tick = 0
    else:
        nice_range = _nice_number(axis_width)
        nice_tick = _nice_number(nice_range / (num_ticks - 1), round_=True)
        axis_start = math.floor(axis_start / nice_tick) * nice_tick
        axis_end = math.ceil(axis_end / nice_tick) * nice_tick

    return axis_start, axis_end, nice_tick

#
# Helpers
#
class Helpers:
    def __init__(self, bb_center):
        self.bb_center = bb_center
        self.center = (0, 0, 0)

    def _center(self, zero=True):
        return self.center if zero else self.bb_center

    def set_position(self, position):
        raise NotImplementedError()

    def set_visibility(self, change):
        raise NotImplementedError()

    def set_center(self, change):
        self.set_position(self._center(change))

#
# Grid helper
#
class Grid(Helpers):
    def __init__(self, bb_center=None, maximum=5, ticks=10, colorCenterLine='#aaa', colorGrid='#ddd'):
        Helpers.__init__(self, bb_center)
        axis_start, axis_end, nice_tick = _nice_bounds(-maximum, maximum, 2 * ticks)
        self.step = nice_tick
        self.size = axis_end - axis_start
        self.grid = GridHelper(self.size, int(self.size / self.step),
                               colorCenterLine=colorCenterLine, colorGrid=colorGrid)
        self.set_center(True)

    def set_position(self, position):
        self.grid.position = position

    def set_visibility(self, change):
        self.grid.visible = change

    def set_rotation(self, rotation):
        self.grid.rotation = rotation


#
# Axes helper
#
class Axes(Helpers):
    """ X, Y and Z axis
    X is red
    Y is green
    Z is blue
    """
    def __init__(self, bb_center, length=1, width=3, display_labels=False):
        Helpers.__init__(self, bb_center)

        self.axes = []
        for vector, color in zip(([length, 0, 0], [0, length, 0], [0, 0, length]), ('red', 'green', 'blue')):
            self.axes.append(LineSegments2(LineSegmentsGeometry(positions=[[self.center,
                                                                            _shift(self.center, vector)]]),
                                           LineMaterial(linewidth=width, color=color)))

        if display_labels:
            # add x, y and z labels
            x_text = make_text("X", [length, 0, 0])
            y_text = make_text("Y", [0, length, 0])
            z_text = make_text("Z", [0, 0, length])

            self.axes.append(x_text)
            self.axes.append(y_text)
            self.axes.append(z_text)

    def set_position(self, position):
        for i in range(3):
            self.axes[i].position = position

    def set_visibility(self, change):
        for i in range(3):
            self.axes[i].visible = change

#
# Custom Material helper
#
class CustomMaterial(ShaderMaterial):
    def __init__(self, typ):
        self.types = {'diffuse': 'c', 'uvTransform': 'm3', 'normalScale': 'v2', 'fogColor': 'c', 'emissive': 'c'}

        shader = ShaderLib[typ]

        fragmentShader = """
        uniform float alpha;
        """
        frag_from = "gl_FragColor = vec4( outgoingLight, diffuseColor.a );"
        frag_to = """
            if ( gl_FrontFacing ) {
                gl_FragColor = vec4( outgoingLight, alpha * diffuseColor.a );
            } else {
                gl_FragColor = vec4( diffuseColor.r, diffuseColor.g, diffuseColor.b, alpha * diffuseColor.a );
            }"""
        fragmentShader += shader["fragmentShader"].replace(frag_from, frag_to)

        vertexShader = shader["vertexShader"]
        uniforms = shader["uniforms"]
        uniforms["alpha"] = dict(value=0.7)

        ShaderMaterial.__init__(self, uniforms=uniforms, vertexShader=vertexShader, fragmentShader=fragmentShader)
        self.lights = True

    @property
    def color(self):
        return self.uniforms["diffuse"]["value"]

    @color.setter
    def color(self, value):
        self.update("diffuse", value)

    @property
    def alpha(self):
        return self.uniforms["alpha"]["value"]

    @alpha.setter
    def alpha(self, value):
        self.update("alpha", value)

    def update(self, key, value):
        uniforms = dict(**self.uniforms)
        if self.types.get(key) is None:
            uniforms[key] = {'value': value}
        else:
            uniforms[key] = {'type': self.types.get(key), 'value': value}
        self.uniforms = uniforms
        self.needsUpdate = True

#
# Bounding Box
#
class BoundingBox:
    def __init__(self, objects, tol=1e-5):
        self.tol = tol

        bbox = reduce(_opt, [self._bbox(obj) for obj in objects])
        self.xmin, self.xmax, self.ymin, self.ymax, self.zmin, self.zmax = bbox
        self.xsize = self.xmax - self.xmin
        self.ysize = self.ymax - self.ymin
        self.zsize = self.zmax - self.zmin
        self.center = (self.xmin + self.xsize / 2.0, self.ymin + self.ysize / 2.0, self.zmin + self.zsize / 2.0)
        self.max = reduce(lambda a, b: max(abs(a), abs(b)), bbox)

    def _max_dist_from_center(self):
        return max([_distance(self.center, v)
                    for v in itertools.product((self.xmin, self.xmax), (self.ymin, self.ymax), (self.zmin, self.zmax))
                   ])

    def _max_dist_from_origin(self):
        return max([np.linalg.norm(v)
                    for v in itertools.product((self.xmin, self.xmax), (self.ymin, self.ymax), (self.zmin, self.zmax))
                   ])

    def _bounding_box(self, obj, tol=1e-5):
        bbox = Bnd_Box()
        bbox.SetGap(self.tol)
        brepbndlib_Add(obj, bbox, True)
        values = bbox.Get()
        return (values[0], values[3], values[1], values[4], values[2], values[5])

    def _bbox(self, objects):
        bb = reduce(_opt, [self._bounding_box(obj) for obj in objects])
        return bb

    def __repr__(self):
        return "[x(%f .. %f), y(%f .. %f), z(%f .. %f)]" % \
               (self.xmin, self.xmax, self.ymin, self.ymax, self.zmin, self.zmax)


class NORMAL(enum.Enum):
    SERVER_SIDE = 1
    CLIENT_SIDE = 2


class JupyterRenderer:
    def __init__(self,
                 size=(640, 480),
                 compute_normals_mode=NORMAL.SERVER_SIDE,
                 default_shape_color=format_color(166, 166, 166), # light grey
                 default_edge_color=format_color(32, 32, 32), # dark grey
                 default_vertex_color=format_color(8, 8, 8), # darker grey
                 pick_color=format_color(232, 176, 36), # orange
                 background_color='white'):
        """ Creates a jupyter renderer.
        size: a tuple (width, height). Must be a square, or shapes will look like deformed
        compute_normals_mode: optional, set to SERVER_SIDE by default. This flag lets you choose the
        way normals are computed. If SERVER_SIDE is selected (default value), then normals
        will be computed by the Tesselator, packed as a python tuple, and send as a json structure
        to the client. If, on the other hand, CLIENT_SIDE is chose, then the computer only compute vertex
        indices, and let the normals be computed by the client (the web js machine embedded in the webrowser).

        * SERVER_SIDE: higher server load, loading time increased, lower client load. Poor performance client will
          choose this option (mobile terminals for instance)
        * CLIENT_SIDE: lower server load, loading time decreased, higher client load. Higher performance clients will
                            choose this option (laptops, desktop machines).
        * default_shape_color
        * default_e1dge_color:
        * default_pick_color:
        * background_color:
        """
        self._default_shape_color = default_shape_color
        self._default_edge_color = default_edge_color
        self._default_vertex_color = default_vertex_color
        self._pick_color = pick_color

        self._background = background_color
        self._background_opacity = 1
        self._size = size
        self._compute_normals_mode = compute_normals_mode

        self._bb = None  # the bounding box, necessary to compute camera position

        # the default camera object
        self._camera_target = [0., 0., 0.]  # the point to look at
        self._camera_position = [0, 0., 100.]  # the camera initial position
        self._camera = None
        self._camera_distance_factor = 6
        self._camera_initial_zoom = 2.5

        # a dictionnary of all the shapes belonging to the renderer
        # each element is a key 'mesh_id:shape'
        self._shapes = {}

        # we save the renderer so that is can be accessed
        self._renderer = None

        # the group of 3d and 2d objects to render
        self._displayed_pickable_objects = Group()

        # the group of other objects (grid, trihedron etc.) that can't be selected
        self._displayed_non_pickable_objects = Group()

        # event manager/selection manager
        self._picker = None

        self._current_shape_selection = None
        self._current_mesh_selection = None
        self._savestate = None

        self._selection_color = format_color(232, 176, 36)

        self._select_callbacks = []  # a list of all functions called after an object is selected

        # UI
        self.layout = Layout(width='auto', height='auto')
        self._toggle_shp_visibility_button = self.create_button("Hide/Show", "Toggle Shape Visibility",
                                                                True, self.toggle_shape_visibility)
        self._shp_properties_button = Dropdown(options=['Compute', 'Inertia', 'Recognize Face', 'Aligned BBox', 'Oriented BBox'],
                                               value='Compute',
                                               description='',
                                               layout=self.layout,
                                               disabled=True)
        self._shp_properties_button.observe(self.on_compute_change)
        self._remove_shp_button = self.create_button("Remove", "Permanently remove the shape from the Scene",
                                                     True, self.remove_shape)
        self._controls = [self.create_checkbox("axes", "Axes", True, self.toggle_axes_visibility),
                          self.create_checkbox("grid", "Grid", True, self.toggle_grid_visibility),
                          self.create_button("Reset View", "Restore default view", False, self._reset),
                          self._shp_properties_button,
                          self._toggle_shp_visibility_button,
                          self._remove_shp_button]
        self.html = HTML("")

    def create_button(self, description, tooltip, disabled, handler):
        button = Button(disabled=disabled, tooltip=tooltip,
                        description=description, layout=self.layout)
        button.on_click(handler)
        return button

    def create_checkbox(self, kind, description, value, handler):
        checkbox = Checkbox(value=value, description=description, layout=self.layout)
        checkbox.observe(handler, "value")
        checkbox.add_class("view_%s" % kind)
        return checkbox

    def remove_shape(self, *kargs):
        self.clicked_obj.visible = not self.clicked_obj.visible
        # remove shape fro mthe mapping dict
        cur_id = self.clicked_obj.name
        del self._shapes[cur_id]
        self._remove_shp_button.disabled = True

    def on_compute_change(self, change):
        if change['type'] == 'change' and change['name'] == 'value':
            selection = change['new']
            output = ""
            if 'Inertia' in selection:
                cog, mass, mass_property = measure_shape_mass_center_of_gravity(self._current_shape_selection)
                # display this point (type gp_Pnt)
                self.DisplayShape([cog])
                output += "<u><b>Center of Gravity</b></u>:<br><b>Xcog=</b>%.3f<br><b>Ycog=</b>%.3f<br><b>Zcog=</b>%.3f<br>" % (cog.X(), cog.Y(), cog.Z())
                output += "<u><b>%s=</b></u>:<b>%.3f</b><br>" % (mass_property, mass)
            elif 'Oriented' in selection:
                center, dim, oobb_shp = get_oriented_boundingbox(self._current_shape_selection)
                self.DisplayShape(oobb_shp,
                                  render_edges=True,
                                  transparency=True,
                                  opacity=0.2,
                                  selectable=False)
                output += "<u><b>OOBB center</b></u>:<br><b>X=</b>%.3f<br><b>Y=</b>%.3f<br><b>Z=</b>%.3f<br>" % (center.X(), center.Y(), center.Z())
                output += "<u><b>OOBB dimensions</b></u>:<br><b>dX=</b>%.3f<br><b>dY=</b>%.3f<br><b>dZ=</b>%.3f<br>" % (dim[0], dim[1], dim[2])
                output += "<u><b>OOBB volume</b></u>:<br><b>V=</b>%.3f<br>" % (dim[0] * dim[1] * dim[2])
            elif 'Aligned' in selection:
                center, dim, albb_shp = get_aligned_boundingbox(self._current_shape_selection)
                self.DisplayShape(albb_shp,
                                  render_edges=True,
                                  transparency=True,
                                  opacity=0.2,
                                  selectable=False)
                output += "<u><b>ABB center</b></u>:<br><b>X=</b>%.3f<br><b>Y=</b>%.3f<br><b>Z=</b>%.3f<br>" % (center.X(), center.Y(), center.Z())
                output += "<u><b>ABB dimensions</b></u>:<br><b>dX=</b>%.3f<br><b>dY=</b>%.3f<br><b>dZ=</b>%.3f<br>" % (dim[0], dim[1], dim[2])
                output += "<u><b>ABB volume</b></u>:<br><b>V=</b>%.3f<br>" % (dim[0] * dim[1] * dim[2])
            elif 'Recognize' in selection:
                # try featrue recognition
                kind, pnt, vec = recognize_face(self._current_shape_selection)
                output += "<u><b>Type</b></u>: %s<br>" % kind
                if kind == "Plane":
                    self.DisplayShape([pnt])
                    output += "<u><b>Properties</b></u>:<br>"
                    output += "<u><b>Point</b></u>:<br><b>X=</b>%.3f<br><b>Y=</b>%.3f<br><b>Z=</b>%.3f<br>" % (pnt.X(), pnt.Y(), pnt.Z())
                    output += "<u><b>Normal</b></u>:<br><b>u=</b>%.3f<br><b>v=</b>%.3f<br><b>w=</b>%.3f<br>" % (vec.X(), vec.Y(), vec.Z())
                elif kind == "Cylinder":
                    self.DisplayShape([pnt])
                    output += "<u><b>Properties</b></u>:<br>"
                    output += "<u><b>Axis point</b></u>:<br><b>X=</b>%.3f<br><b>Y=</b>%.3f<br><b>Z=</b>%.3f<br>" % (pnt.X(), pnt.Y(), pnt.Z())
                    output += "<u><b>Axis direction</b></u>:<br><b>u=</b>%.3f<br><b>v=</b>%.3f<br><b>w=</b>%.3f<br>" % (vec.X(), vec.Y(), vec.Z())
            self.html.value = output

    def toggle_shape_visibility(self, *kargs):
        self.clicked_obj.visible = not self.clicked_obj.visible

    def toggle_axes_visibility(self, change):
        self.axes.set_visibility(_bool_or_new(change))

    def toggle_grid_visibility(self, change):
        self.horizontal_grid.set_visibility(_bool_or_new(change))
        self.vertical_grid.set_visibility(_bool_or_new(change))

    def click(self, value):
        """ called whenever a shape  or edge is clicked
        """
        obj = value.owner.object
        self.clicked_obj = obj
        if self._current_mesh_selection != obj:
            if self._current_mesh_selection is not None:
                self._current_mesh_selection.material.color = self._current_selection_material_color
                self._current_mesh_selection.material.transparent = False
                self._current_mesh_selection = None
                self._current_selection_material_color = None
                self._shp_properties_button.value = "Compute"
                self._shp_properties_button.disabled = True
                self._toggle_shp_visibility_button.disabled = True
                self._remove_shp_button.disabled = True
                self._current_shape_selection = None
            if obj is not None:
                self._shp_properties_button.disabled = False
                self._toggle_shp_visibility_button.disabled = False
                self._remove_shp_button.disabled = False
                id_clicked = obj.name  # the mesh id clicked
                self._current_mesh_selection = obj
                self._current_selection_material_color = obj.material.color
                obj.material.color = self._selection_color
                # selected part becomes transparent
                obj.material.transparent = True
                obj.material.opacity = 0.5
                # get the shape from this mesh id
                selected_shape = self._shapes[id_clicked]
                html_value = "<b>Shape type:</b> %s<br>" % get_type_as_string(selected_shape)
                html_value += "<b>Shape id:</b> %s<br>" % id_clicked
                self.html.value = html_value
                self._current_shape_selection = selected_shape
            else:
                self.html.value = "<b>Shape type:</b> None<br><b>Shape id:</b> None"
            # then execute calbacks
            for callback in self._select_callbacks:
                callback(self._current_shape_selection)

    def register_select_callback(self, callback):
        """ Adds a callback that will be called each time a shape is selected
        """
        if not callable(callback):
            raise AssertionError("You must provide a callable to register the callback")
        else:
            self._select_callbacks.append(callback)

    def unregister_callback(self, callback):
        """ Remove a callback from the callback list
        """
        if callback not in self._select_callbacks:
            raise AssertionError("This callback is not registered")
        else:
            self._select_callbacks.remove(callback)

    def GetSelectedShape(self):
        """ Returns the selected shape
        """
        return self._current_shape_selection

    def DisplayShapeAsSVG(self,
                          shp,
                          export_hidden_edges=True,
                          location=gp_Pnt(0, 0, 0),
                          direction=gp_Dir(1, 1, 1),
                          color="black",
                          line_width=0.5):
        svg_string = export_shape_to_svg(shp, export_hidden_edges=export_hidden_edges,
                                         location=location, direction=direction,
                                         color=color, line_width=line_width,
                                         margin_left=0, margin_top=0)
        svg = SVG(data=svg_string)
        display(svg)

    def DisplayShape(self,
                     shp,
                     shape_color=None,
                     render_edges=False,
                     edge_color=None,
                     edge_deflection=0.05,
                     vertex_color=None,
                     quality=1.0,
                     transparency=False,
                     opacity=1.,
                     topo_level='default',
                     update=False,
                     selectable=True):
        """ Displays a topods_shape in the renderer instance.
        shp: the TopoDS_Shape to render
        shape_color: the shape color, in html corm, eg '#abe000'
        render_edges: optional, False by default. If True, compute and dislay all
                      edges as a linear interpolation of segments.
        edge_color: optional, black by default. The color used for edge rendering,
                    in html form eg '#ff00ee'
        edge_deflection: optional, 0.05 by default
        vertex_color: optional
        quality: optional, 1.0 by default. If set to something lower than 1.0,
                      mesh will be more precise. If set to something higher than 1.0,
                      mesh will be less precise, i.e. lower numer of triangles.
        transparency: optional, False by default (opaque).
        opacity: optional, float, by default to 1 (opaque). if transparency is set to True,
                 0. is fully opaque, 1. is fully transparent.
        topo_level: "default" by default. The value should be either "compound", "shape", "vertex".
        update: optional, False by default. If True, render all the shapes.
        selectable: if True, can be doubleclicked from the 3d window
        """
        if edge_color is None:
            edge_color = self._default_edge_color
        if shape_color is None:
            shape_color = self._default_shape_color
        if vertex_color is None:
            vertex_color = self._default_vertex_color

        output = [] # a list of all geometries created from the shape
        # is it list of gp_Pnt ?
        if isinstance(shp, list) and isinstance(shp[0], gp_Pnt):
            result = self.AddVerticesToScene(shp, vertex_color)
            output.append(result)
        # or a 1d element such as edge or wire ?
        elif is_wire(shp) or is_edge(shp):
            result = self.AddCurveToScene(shp, edge_color, edge_deflection)
            output.append(result)
        elif topo_level != "default":
            t = TopologyExplorer(shp)
            map_type_and_methods = {"Solid": t.solids, "Face": t.faces, "Shell": t.shells,
                                    "Compound": t.compounds, "Compsolid": t.comp_solids}
            for subshape in map_type_and_methods[topo_level]():
                result = self.AddShapeToScene(subshape, shape_color, render_edges, edge_color,
                                              vertex_color, quality, transparency, opacity)
                output.append(result)
        else:
            result = self.AddShapeToScene(shp, shape_color, render_edges,
                                          edge_color, vertex_color, quality,
                                          transparency, opacity)
            output.append(result)

        if selectable:# Add geometries to pickable or non pickable objects
            for elem in output:
                self._displayed_pickable_objects.add(elem)

        if update:
            self.Display()

    def AddVerticesToScene(self, pnt_list, vertex_color, vertex_width=5):
        """ shp is a list of gp_Pnt
        """
        vertices_list = []  # will be passed to pythreejs
        BB = BRep_Builder()
        compound = TopoDS_Compound()
        BB.MakeCompound(compound)

        for vertex in pnt_list:
            vertex_to_add = BRepBuilderAPI_MakeVertex(vertex).Shape()
            BB.Add(compound, vertex_to_add)
            vertices_list.append([vertex.X(), vertex.Y(), vertex.Z()])

        # map the Points and the AIS_PointCloud
        # and to the dict of shapes, to have a mapping between meshes and shapes
        point_cloud_id = "%s" % uuid.uuid4().hex
        self._shapes[point_cloud_id] = compound

        vertices_list = np.array(vertices_list, dtype=np.float32)
        attributes = {"position": BufferAttribute(vertices_list, normalized=False)}
        mat = PointsMaterial(color=vertex_color, sizeAttenuation=True, size=vertex_width)
        geom = BufferGeometry(attributes=attributes)
        points = Points(geometry=geom, material=mat, name=point_cloud_id)
        return points

    def AddCurveToScene(self, shp, edge_color, deflection):
        """ shp is either a TopoDS_Wire or a TopodS_Edge.
        """
        if is_edge(shp):
            pnts = discretize_edge(shp, deflection)
        elif is_wire(shp):
            pnts = discretize_wire(shp, deflection)
        np_edge_vertices = np.array(pnts, dtype=np.float32)
        np_edge_indices = np.arange(np_edge_vertices.shape[0], dtype=np.uint32)
        edge_geometry = BufferGeometry(attributes={
            'position': BufferAttribute(np_edge_vertices),
            'index': BufferAttribute(np_edge_indices)
        })
        edge_material = LineBasicMaterial(color=edge_color, linewidth=1)

        # and to the dict of shapes, to have a mapping between meshes and shapes
        edge_id = "%s" % uuid.uuid4().hex
        self._shapes[edge_id] = shp

        edge_line = Line(geometry=edge_geometry,
                         material=edge_material,
                         name=edge_id)

        # and to the dict of shapes, to have a mapping between meshes and shapes
        edge_id = "%s" % uuid.uuid4().hex
        self._shapes[edge_id] = shp

        return edge_line


    def AddShapeToScene(self,
                        shp,
                        shape_color=None,  # the default
                        render_edges=False,
                        edge_color=None,
                        vertex_color=None,
                        quality=1.0,
                        transparency=False,
                        opacity=1.):
        # first, compute the tesselation
        tess = ShapeTesselator(shp)
        tess.Compute(compute_edges=render_edges,
                     mesh_quality=quality,
                     parallel=True)
        # get vertices and normals
        vertices_position = tess.GetVerticesPositionAsTuple()

        number_of_triangles = tess.ObjGetTriangleCount()
        number_of_vertices = len(vertices_position)

        # number of vertices should be a multiple of 3
        if number_of_vertices % 3 != 0:
            raise AssertionError("Wrong number of vertices")
        if number_of_triangles * 9 != number_of_vertices:
            raise AssertionError("Wrong number of triangles")

        # then we build the vertex and faces collections as numpy ndarrays
        np_vertices = np.array(vertices_position, dtype='float32').reshape(int(number_of_vertices / 3), 3)
        # Note: np_faces is just [0, 1, 2, 3, 4, 5, ...], thus arange is used
        np_faces = np.arange(np_vertices.shape[0], dtype='uint32')

        # set geometry properties
        buffer_geometry_properties = {'position': BufferAttribute(np_vertices),
                                      'index': BufferAttribute(np_faces)}
        if self._compute_normals_mode == NORMAL.SERVER_SIDE:
            # get the normal list, converts to a numpy ndarray. This should not raise
            # any issue, since normals have been computed by the server, and are available
            # as a list of floats
            np_normals = np.array(tess.GetNormalsAsTuple(), dtype='float32').reshape(-1, 3)
            # quick check
            if np_normals.shape != np_vertices.shape:
                raise AssertionError("Wrong number of normals/shapes")
            buffer_geometry_properties['normal'] = BufferAttribute(np_normals)

        # build a BufferGeometry instance
        shape_geometry = BufferGeometry(attributes=buffer_geometry_properties)

        # if the client has to render normals, add the related js instructions
        if self._compute_normals_mode == NORMAL.CLIENT_SIDE:
            shape_geometry.exec_three_obj_method('computeVertexNormals')

        # then a default material
        shp_material = self._material(shape_color, transparent=transparency, opacity=opacity)

        # and to the dict of shapes, to have a mapping between meshes and shapes
        mesh_id = "%s" % uuid.uuid4().hex
        self._shapes[mesh_id] = shp

        # finally create the mesh
        shape_mesh = Mesh(geometry=shape_geometry,
                          material=shp_material,
                          name=mesh_id)

        # edge rendering, if set to True
        if render_edges:
            edges = list(map(lambda i_edge: [tess.GetEdgeVertex(i_edge, i_vert) for i_vert in range(tess.ObjEdgeGetVertexCount(i_edge))], range(tess.ObjGetEdgeCount())))
            edge_list = _flatten(list(map(_explode, edges)))
            lines = LineSegmentsGeometry(positions=edge_list)
            mat = LineMaterial(linewidth=1, color=edge_color)
            edge_lines = LineSegments2(lines, mat)
            self._displayed_non_pickable_objects.add(edge_lines)

        return shape_mesh

    def _scale(self, vec):
        r = self._bb._max_dist_from_center() * self._camera_distance_factor
        n = np.linalg.norm(vec)
        new_vec = [v / n * r for v in vec]
        return new_vec

    def _material(self, color, transparent=False, opacity=1.0):
        #material = MeshPhongMaterial()
        material = CustomMaterial("standard")
        material.color = color
        material.clipping = True
        material.side = "DoubleSide"
        material.polygonOffset = True
        material.polygonOffsetFactor = 1
        material.polygonOffsetUnits = 1
        material.transparent = transparent
        material.opacity = opacity
        material.update("metalness", 0.3)
        material.update("roughness", 0.8)
        return material

    def EraseAll(self):
        self._shapes = {}
        self._displayed_pickable_objects = Group()
        self._current_shape_selection = None
        self._current_mesh_selection = None
        self._current_selection_material = None
        self._renderer.scene = Scene(children=[])

    def Display(self, position=None, rotation=None):
        # Get the overall bounding box
        if self._shapes:
            self._bb = BoundingBox([self._shapes.values()])
        else:  # if nothing registered yet, create a fake bb
            self._bb = BoundingBox([[BRepPrimAPI_MakeSphere(5.).Shape()]])
        bb_max = self._bb.max
        orbit_radius = 1.5 * self._bb._max_dist_from_center()

        # Set up camera
        camera_target = self._bb.center
        camera_position = _add(self._bb.center,
                               self._scale([1, 1, 1] if position is None else self._scale(position)))
        camera_zoom = self._camera_initial_zoom

        self._camera = CombinedCamera(position=camera_position,
                                      width=self._size[0], height=self._size[1])
        self._camera.up = (0.0, 0.0, 1.0)
        self._camera.mode = 'orthographic'
        self._camera_target = camera_target
        self._camera.position = camera_position
        if rotation is not None:
            self._camera.rotation = rotation
        # Set up lights in every of the 8 corners of the global bounding box
        positions = list(itertools.product(*[(-orbit_radius, orbit_radius)] * 3))
        key_lights = [DirectionalLight(color='white',
                                       position=pos,
                                       intensity=0.5) for pos in positions]
        ambient_light = AmbientLight(intensity=0.1)

        # Set up Helpers
        self.axes = Axes(bb_center=self._bb.center, length=bb_max * 1.1)
        self.horizontal_grid = Grid(bb_center=self._bb.center, maximum=bb_max,
                                    colorCenterLine='#aaa', colorGrid='#ddd')
        self.vertical_grid = Grid(bb_center=self._bb.center, maximum=bb_max,
                                  colorCenterLine='#aaa', colorGrid='#ddd')
        # Set up scene
        environment = self.axes.axes + key_lights + [ambient_light,
                                                     self.horizontal_grid.grid,
                                                     self.vertical_grid.grid,
                                                     self._camera]

        scene_shp = Scene(children=[self._displayed_pickable_objects,
                                    self._displayed_non_pickable_objects] + environment)

        # Set up Controllers
        self._controller = OrbitControls(controlling=self._camera,
                                         target=camera_target,
                                         target0=camera_target)
        # Update controller to instantiate camera position
        self._camera.zoom = camera_zoom
        self._update()

        # setup Picker
        self._picker = Picker(controlling=self._displayed_pickable_objects, event='dblclick')
        self._picker.observe(self.click)

        self._renderer = Renderer(camera=self._camera,
                                  background=self._background,
                                  background_opacity=self._background_opacity,
                                  scene=scene_shp,
                                  controls=[self._controller, self._picker],
                                  width=self._size[0],
                                  height=self._size[1],
                                  antialias=True)

        # set rotation and position for each grid
        self.horizontal_grid.set_position((0, 0, 0))
        self.horizontal_grid.set_rotation((math.pi / 2.0, 0, 0, "XYZ"))

        self.vertical_grid.set_position((0, - bb_max, 0))

        self._savestate = (self._camera.rotation, self._controller.target)

        # then display both 3d widgets and webui
        display(HBox([VBox([HBox(self._controls), self._renderer]),
                      self.html]))


    def ExportToHTML(self, filename):
        embed.embed_minimal_html(filename, views=self._renderer, title='pythonocc')

    def _reset(self, *kargs):
        self._camera.rotation, self._controller.target = self._savestate
        self._camera.position = _add(self._bb.center, self._scale((1, 1, 1)))
        self._camera.zoom = self._camera_initial_zoom
        self._update()

    def _update(self):
        self._controller.exec_three_obj_method('update')

    def __repr__(self):
        self.Display()
        return ""



from enum import IntEnum
from typing import overload, NewType, Optional, Tuple


Standard_Address = NewType('Standard_Address', None)
Standard_Boolean = NewType('Standard_Boolean', bool)
Standard_Byte = NewType('Standard_Byte', str)
Standard_CString = NewType('Standard_CString', str)
Standard_Character = NewType('Standard_Character', str)
#the following typedef cannot be wrapped as is
Standard_ErrorHandlerCallback = NewType('Standard_ErrorHandlerCallback', Any)
Standard_ExtCharacter = NewType('Standard_ExtCharacter', str)
Standard_ExtString = NewType('Standard_ExtString', str)
#the following typedef cannot be wrapped as is
Standard_IStream = NewType('Standard_IStream', Any)
Standard_Integer = NewType('Standard_Integer', int)
#the following typedef cannot be wrapped as is
Standard_OStream = NewType('Standard_OStream', Any)
Standard_PByte = NewType('Standard_PByte', Standard_Byte)
Standard_PCharacter = NewType('Standard_PCharacter', str)
Standard_PErrorHandler = NewType('Standard_PErrorHandler', Standard_ErrorHandler)
Standard_PExtCharacter = NewType('Standard_PExtCharacter', str)
Standard_Real = NewType('Standard_Real', float)
#the following typedef cannot be wrapped as is
Standard_SStream = NewType('Standard_SStream', Any)
Standard_ShortReal = NewType('Standard_ShortReal', float)
Standard_Size = NewType('Standard_Size', int)
Standard_ThreadId = NewType('Standard_ThreadId', Standard_Size)
#the following typedef cannot be wrapped as is
Standard_Time = NewType('Standard_Time', Any)
Standard_UUID = NewType('Standard_UUID', str)
Standard_Utf16Char = NewType('Standard_Utf16Char', str)
Standard_Utf32Char = NewType('Standard_Utf32Char', str)
Standard_Utf8Char = NewType('Standard_Utf8Char', str)
Standard_Utf8UChar = NewType('Standard_Utf8UChar', str)
Standard_WideChar = NewType('Standard_WideChar', str)
int16_t = NewType('int16_t', int)
int32_t = NewType('int32_t', int)
int64_t = NewType('int64_t', int)
int8_t = NewType('int8_t', int)
uint16_t = NewType('uint16_t', int)
uint32_t = NewType('uint32_t', int)
uint64_t = NewType('uint64_t', int)
uint8_t = NewType('uint8_t', int)

class Standard_HandlerStatus(IntEnum):
	Standard_HandlerVoid: int = ...
	Standard_HandlerJumped: int = ...
	Standard_HandlerProcessed: int = ...
Standard_HandlerVoid = Standard_HandlerStatus.Standard_HandlerVoid
Standard_HandlerJumped = Standard_HandlerStatus.Standard_HandlerJumped
Standard_HandlerProcessed = Standard_HandlerStatus.Standard_HandlerProcessed

class standard:
	@staticmethod
	def Allocate(aSize: int) -> None: ...
	@staticmethod
	def AllocateAligned(theSize: int, theAlign: int) -> None: ...
	@staticmethod
	def Purge() -> int: ...
	@staticmethod
	def Reallocate(aStorage: None, aNewSize: int) -> None: ...

class Standard_ArrayStreamBuffer():
	pass

class Standard_Condition:
	def Check(self) -> False: ...
	def CheckReset(self) -> False: ...
	def Reset(self) -> None: ...
	def Set(self) -> None: ...
	@overload
	def Wait(self) -> None: ...

class Standard_ErrorHandler:
	def __init__(self) -> None: ...
	def Catches(self, aType: Standard_Type) -> bool: ...
	def Destroy(self) -> None: ...
	def Error(self) -> Standard_Failure: ...
	@staticmethod
	def IsInTryBlock() -> bool: ...
	@staticmethod
	def LastCaughtError() -> Standard_Failure: ...
	def Unlink(self) -> None: ...

class Standard_GUID:
	@overload
	def __init__(self) -> None: ...
	@overload
	def __init__(self, aGuid: str) -> None: ...
	@overload
	def __init__(self, aGuid: Standard_ExtString) -> None: ...
	@overload
	def __init__(self, a32b: int, a16b1: Standard_ExtCharacter, a16b2: Standard_ExtCharacter, a16b3: Standard_ExtCharacter, a8b1: str, a8b2: str, a8b3: str, a8b4: str, a8b5: str, a8b6: str) -> None: ...
	@overload
	def __init__(self, aGuid: Standard_UUID) -> None: ...
	@overload
	def __init__(self, aGuid: Standard_GUID) -> None: ...
	@overload
	def Assign(self, uid: Standard_GUID) -> None: ...
	@overload
	def Assign(self, uid: Standard_UUID) -> None: ...
	@staticmethod
	def CheckGUIDFormat(aGuid: str) -> bool: ...
	def Hash(self, Upper: int) -> int: ...
	@staticmethod
	def HashCode(theGUID: Standard_GUID, theUpperBound: int) -> int: ...
	@staticmethod
	def IsEqual(string1: Standard_GUID, string2: Standard_GUID) -> bool: ...
	def IsNotSame(self, uid: Standard_GUID) -> bool: ...
	def IsSame(self, uid: Standard_GUID) -> bool: ...
	def ToCString(self, aStrGuid: Standard_PCharacter) -> None: ...
	def ToExtString(self, aStrGuid: Standard_PExtCharacter) -> None: ...
	def ToUUID(self) -> Standard_UUID: ...

class Standard_MMgrRoot:
	def Allocate(self, theSize: int) -> None: ...
	def Free(self, thePtr: None) -> None: ...
	def Purge(self, isDestroyed: Optional[bool] = False) -> int: ...
	def Reallocate(self, thePtr: None, theSize: int) -> None: ...

class Standard_Static_Assert:
	@staticmethod
	def assert_ok() -> None: ...

class Standard_Transient:
	@overload
	def __init__(self) -> None: ...
	def DecrementRefCounter(self) -> int: ...
	def Delete(self) -> None: ...
	def DynamicType(self) -> Standard_Type: ...
	def GetRefCount(self) -> int: ...
	def IncrementRefCounter(self) -> None: ...
	@overload
	def IsInstance(self, theType: Standard_Type) -> bool: ...
	@overload
	def IsInstance(self, theTypeName: str) -> bool: ...
	@overload
	def IsKind(self, theType: Standard_Type) -> bool: ...
	@overload
	def IsKind(self, theTypeName: str) -> bool: ...
	def This(self) -> Standard_Transient: ...
	@staticmethod
	def get_type_descriptor() -> Standard_Type: ...
	@staticmethod
	def get_type_name() -> str: ...

class Standard_Failure(Standard_Transient):
	@overload
	def __init__(self) -> None: ...
	@overload
	def __init__(self, f: Standard_Failure) -> None: ...
	@overload
	def __init__(self, aString: str) -> None: ...
	@staticmethod
	def Caught() -> Standard_Failure: ...
	def GetMessageString(self) -> str: ...
	def Jump(self) -> None: ...
	@staticmethod
	def NewInstance(aMessage: str) -> Standard_Failure: ...
	@overload
	@staticmethod
	def Raise(aMessage: Optional[str] = "") -> None: ...
	@overload
	@staticmethod
	def Raise(aReason: Standard_SStream) -> None: ...
	@overload
	def Reraise(self) -> None: ...
	@overload
	def Reraise(self, aMessage: str) -> None: ...
	@overload
	def Reraise(self, aReason: Standard_SStream) -> None: ...
	def SetMessageString(self, aMessage: str) -> None: ...

class Standard_MMgrOpt(Standard_MMgrRoot):
	def __init__(self, aClear: Optional[bool] = True, aMMap: Optional[bool] = True, aCellSize: Optional[int] = 200, aNbPages: Optional[int] = 10000, aThreshold: Optional[int] = 40000) -> None: ...
	def Allocate(self, aSize: int) -> None: ...
	def Free(self, thePtr: None) -> None: ...
	def Purge(self, isDestroyed: bool) -> int: ...
	def Reallocate(self, thePtr: None, theSize: int) -> None: ...

class Standard_MMgrRaw(Standard_MMgrRoot):
	def __init__(self, aClear: Optional[bool] = False) -> None: ...
	def Allocate(self, aSize: int) -> None: ...
	def Free(self, thePtr: None) -> None: ...
	def Reallocate(self, thePtr: None, theSize: int) -> None: ...

class Standard_MMgrTBBalloc(Standard_MMgrRoot):
	def __init__(self, aClear: Optional[bool] = False) -> None: ...
	def Allocate(self, aSize: int) -> None: ...
	def Free(self, thePtr: None) -> None: ...
	def Reallocate(self, thePtr: None, theSize: int) -> None: ...

class Standard_OutOfMemory(Standard_ProgramError):
	def __init__(self, theMessage: Optional[str] = 0) -> None: ...
	def GetMessageString(self) -> str: ...
	@staticmethod
	def NewInstance(theMessage: Optional[str] = "") -> Standard_OutOfMemory: ...
	@overload
	@staticmethod
	def Raise(theMessage: Optional[str] = "") -> None: ...
	@overload
	@staticmethod
	def Raise(theMessage: Standard_SStream) -> None: ...
	def SetMessageString(self, aMessage: str) -> None: ...

class Standard_Persistent(Standard_Transient):
	def __init__(self) -> None: ...
	def GetTypeNum(self) -> int: ...
	def SetTypeNum(self, value: int) -> None: ...

class Standard_Type(Standard_Transient):
	def Name(self) -> str: ...
	def Parent(self) -> Standard_Type: ...
	@staticmethod
	def Register(theSystemName: str, theName: str, theSize: int, theParent: Standard_Type) -> Standard_Type: ...
	def Size(self) -> int: ...
	@overload
	def SubType(self, theOther: Standard_Type) -> bool: ...
	@overload
	def SubType(self, theOther: str) -> bool: ...
	def SystemName(self) -> str: ...

#classnotwrapped
class Standard_AncestorIterator: ...

#classnotwrapped
class Standard_Static_Assert: ...

#classnotwrapped
class Standard_CLocaleSentry: ...

#classnotwrapped
class Standard_Mutex: ...

#classnotwrapped
class Standard_ReadLineBuffer: ...

#classnotwrapped
class Standard_ProgramError: ...

#classnotwrapped
class Standard_ReadBuffer: ...

# harray1 classes
# harray2 classes
# hsequence classes

standard_Allocate = standard.Allocate
standard_AllocateAligned = standard.AllocateAligned
standard_Purge = standard.Purge
standard_Reallocate = standard.Reallocate
Standard_ErrorHandler_IsInTryBlock = Standard_ErrorHandler.IsInTryBlock
Standard_ErrorHandler_LastCaughtError = Standard_ErrorHandler.LastCaughtError
Standard_GUID_CheckGUIDFormat = Standard_GUID.CheckGUIDFormat
Standard_GUID_HashCode = Standard_GUID.HashCode
Standard_GUID_IsEqual = Standard_GUID.IsEqual
Standard_Transient_get_type_descriptor = Standard_Transient.get_type_descriptor
Standard_Transient_get_type_name = Standard_Transient.get_type_name
Standard_Failure_Caught = Standard_Failure.Caught
Standard_Failure_NewInstance = Standard_Failure.NewInstance
Standard_Failure_Raise = Standard_Failure.Raise
Standard_Failure_Raise = Standard_Failure.Raise
Standard_OutOfMemory_NewInstance = Standard_OutOfMemory.NewInstance
Standard_OutOfMemory_Raise = Standard_OutOfMemory.Raise
Standard_OutOfMemory_Raise = Standard_OutOfMemory.Raise
Standard_Type_Register = Standard_Type.Register



# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

"""
TopClass module, see official documentation at
https://www.opencascade.com/doc/occt-7.4.0/refman/html/package_topclass.html
"""

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _TopClass
else:
    import _TopClass

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


class SwigPyIterator(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _TopClass.delete_SwigPyIterator

    def value(self) -> "PyObject *":
        return _TopClass.SwigPyIterator_value(self)

    def incr(self, n: "size_t"=1) -> "swig::SwigPyIterator *":
        return _TopClass.SwigPyIterator_incr(self, n)

    def decr(self, n: "size_t"=1) -> "swig::SwigPyIterator *":
        return _TopClass.SwigPyIterator_decr(self, n)

    def distance(self, x: "SwigPyIterator") -> "ptrdiff_t":
        return _TopClass.SwigPyIterator_distance(self, x)

    def equal(self, x: "SwigPyIterator") -> "bool":
        return _TopClass.SwigPyIterator_equal(self, x)

    def copy(self) -> "swig::SwigPyIterator *":
        return _TopClass.SwigPyIterator_copy(self)

    def next(self) -> "PyObject *":
        return _TopClass.SwigPyIterator_next(self)

    def __next__(self) -> "PyObject *":
        return _TopClass.SwigPyIterator___next__(self)

    def previous(self) -> "PyObject *":
        return _TopClass.SwigPyIterator_previous(self)

    def advance(self, n: "ptrdiff_t") -> "swig::SwigPyIterator *":
        return _TopClass.SwigPyIterator_advance(self, n)

    def __eq__(self, x: "SwigPyIterator") -> "bool":
        return _TopClass.SwigPyIterator___eq__(self, x)

    def __ne__(self, x: "SwigPyIterator") -> "bool":
        return _TopClass.SwigPyIterator___ne__(self, x)

    def __iadd__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator &":
        return _TopClass.SwigPyIterator___iadd__(self, n)

    def __isub__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator &":
        return _TopClass.SwigPyIterator___isub__(self, n)

    def __add__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator *":
        return _TopClass.SwigPyIterator___add__(self, n)

    def __sub__(self, *args) -> "ptrdiff_t":
        return _TopClass.SwigPyIterator___sub__(self, *args)
    def __iter__(self):
        return self

# Register SwigPyIterator in _TopClass:
_TopClass.SwigPyIterator_swigregister(SwigPyIterator)


def _dumps_object(klass):
    """ Overwrite default string output for any wrapped object.
    By default, __repr__ method returns something like:
    <OCC.Core.TopoDS.TopoDS_Shape; proxy of <Swig Object of type 'TopoDS_Shape *' at 0x02BB0758> >
    This is too much verbose.
    We prefer :
    <class 'gp_Pnt'>
    or
    <class 'TopoDS_Shape'>
    """
    klass_name = str(klass.__class__).split(".")[3].split("'")[0]
    repr_string = "<class '" + klass_name + "'"
# for TopoDS_Shape, we also look for the base type
    if klass_name == "TopoDS_Shape":
        if klass.IsNull():
            repr_string += ": Null>"
            return repr_string
        st = klass.ShapeType()
        types = {OCC.Core.TopAbs.TopAbs_VERTEX: "Vertex",
                 OCC.Core.TopAbs.TopAbs_SOLID: "Solid",
                 OCC.Core.TopAbs.TopAbs_EDGE: "Edge",
                 OCC.Core.TopAbs.TopAbs_FACE: "Face",
                 OCC.Core.TopAbs.TopAbs_SHELL: "Shell",
                 OCC.Core.TopAbs.TopAbs_WIRE: "Wire",
                 OCC.Core.TopAbs.TopAbs_COMPOUND: "Compound",
                 OCC.Core.TopAbs.TopAbs_COMPSOLID: "Compsolid"}
        repr_string += "; Type:%s" % types[st]        
    elif hasattr(klass, "IsNull"):
        if klass.IsNull():
            repr_string += "; Null"
    repr_string += ">"
    return repr_string


def process_exception(error: "Standard_Failure", method_name: "std::string", class_name: "std::string") -> "void":
    return _TopClass.process_exception(error, method_name, class_name)

from six import with_metaclass
import warnings
from OCC.Wrapper.wrapper_utils import Proxy, deprecated

import OCC.Core.Standard
import OCC.Core.NCollection
import OCC.Core.TopoDS
import OCC.Core.Message
import OCC.Core.TCollection
import OCC.Core.OSD
import OCC.Core.TColStd
import OCC.Core.TopAbs
import OCC.Core.TopLoc
import OCC.Core.gp

from enum import IntEnum
from OCC.Core.Exception import *



class TopClass_SolidExplorer(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    def CurrentFace(self, *args) -> "TopoDS_Face":
        r"""
        Returns the current face.

        Returns
        -------
        TopoDS_Face

        """
        return _TopClass.TopClass_SolidExplorer_CurrentFace(self, *args)

    def InitFace(self, *args) -> "void":
        r"""
        Starts an exploration of the faces.

        Returns
        -------
        None

        """
        return _TopClass.TopClass_SolidExplorer_InitFace(self, *args)

    def InitShell(self, *args) -> "void":
        r"""
        Starts an exploration of the shells.

        Returns
        -------
        None

        """
        return _TopClass.TopClass_SolidExplorer_InitShell(self, *args)

    def MoreFaces(self, *args) -> "Standard_Boolean":
        r"""
        Returns true if there is a current face.

        Returns
        -------
        bool

        """
        return _TopClass.TopClass_SolidExplorer_MoreFaces(self, *args)

    def MoreShells(self, *args) -> "Standard_Boolean":
        r"""
        Returns true if there is a current shell.

        Returns
        -------
        bool

        """
        return _TopClass.TopClass_SolidExplorer_MoreShells(self, *args)

    def NextFace(self, *args) -> "void":
        r"""
        Sets the explorer to the next face and returns false if there are no more wires.

        Returns
        -------
        None

        """
        return _TopClass.TopClass_SolidExplorer_NextFace(self, *args)

    def NextShell(self, *args) -> "void":
        r"""
        Sets the explorer to the next shell and returns false if there are no more wires.

        Returns
        -------
        None

        """
        return _TopClass.TopClass_SolidExplorer_NextShell(self, *args)

    def OtherSegment(self, *args) -> "void":
        r"""
        Returns in <l>, <par> a segment having at least one intersection with the shape boundary to compute intersections. //! the first call to this method returns a line which point to a point of the first face of the shape. the second call provide a line to the second face and so on. //! if the method is called n times on a shape with f faces (n>f) the line point to other points on the face 1,2,3 ... n.

        Parameters
        ----------
        P: gp_Pnt
        L: gp_Lin

        Returns
        -------
        Par: float

        """
        return _TopClass.TopClass_SolidExplorer_OtherSegment(self, *args)

    def Reject(self, *args) -> "Standard_Boolean":
        r"""
        Should return true if the point is outside a bounding volume of the shape.

        Parameters
        ----------
        P: gp_Pnt

        Returns
        -------
        bool

        """
        return _TopClass.TopClass_SolidExplorer_Reject(self, *args)

    def RejectFace(self, *args) -> "Standard_Boolean":
        r"""
        Returns true if the face bounding volume does not intersect the segment.

        Parameters
        ----------
        L: gp_Lin
        Par: float

        Returns
        -------
        bool

        """
        return _TopClass.TopClass_SolidExplorer_RejectFace(self, *args)

    def RejectShell(self, *args) -> "Standard_Boolean":
        r"""
        Returns true if the shell bounding volume does not intersect the segment.

        Parameters
        ----------
        L: gp_Lin
        Par: float

        Returns
        -------
        bool

        """
        return _TopClass.TopClass_SolidExplorer_RejectShell(self, *args)

    def Segment(self, *args) -> "void":
        r"""
        Returns in <l>, <par> a segment having at least one intersection with the shape boundary to compute intersections.

        Parameters
        ----------
        P: gp_Pnt
        L: gp_Lin

        Returns
        -------
        Par: float

        """
        return _TopClass.TopClass_SolidExplorer_Segment(self, *args)

    __repr__ = _dumps_object

    __swig_destroy__ = _TopClass.delete_TopClass_SolidExplorer

# Register TopClass_SolidExplorer in _TopClass:
_TopClass.TopClass_SolidExplorer_swigregister(TopClass_SolidExplorer)





